use std::collections::{BTreeMap, HashMap};

/// An item which stores known certificates. For convenience, this trait is implemented for
/// `std::collections::HashMap<String, String>` and `std::collections::BTreeMap<String, String>`
pub trait CertificateStore: Send + Sync {
    /// Retrieves a certificate fingerprint using the hostname as it's key if 
    /// it has previously been saved
    fn get_certificate(&self, host: &str) -> Option<String>;
    /// Inserts a certificate fingerprint with the hostname as the key
    fn insert_certificate(&mut self, host: &str, fingerprint: &str) -> Option<String>;
    /// Returns true if a fingerprint has been stored for this host
    fn contains_certificate(&self, host: &str) -> bool;
}

impl CertificateStore for HashMap<String, String> {
    fn get_certificate(&self, host: &str) -> Option<String> {
        self.get(host).cloned()
    }

    fn insert_certificate(&mut self, host: &str, fingerprint: &str) -> Option<String> {
        self.insert(host.to_string(), fingerprint.to_string())
    }

    fn contains_certificate(&self, host: &str) -> bool {
        self.contains_key(host)
    }
}

impl CertificateStore for BTreeMap<String, String> {
    fn get_certificate(&self, host: &str) -> Option<String> {
        self.get(host).cloned()
    }

    fn insert_certificate(&mut self, host: &str, fingerprint: &str) -> Option<String> {
        self.insert(host.to_string(), fingerprint.to_string())
    }

    fn contains_certificate(&self, host: &str) -> bool {
        self.contains_key(host)
    }
}
